from gjt.misc.connect_to_websocket import connect_to_websocket
from gjt.misc.get_error_data import get_error_data
from gjt.misc.get_network_data import get_network_data
from gjt.account.login_to_account import login_to_account
from gjt.misc.skips_calculator import skips_calculator
from gjt.map.switch_kingdoms import switch_kingdoms
__all__ = ["connect_to_websocket", "get_error_data", "get_network_data", "login_to_account", "skips_calculator", "switch_kingdoms"]
__version__ = "2.2.4"


