from loguru import logger
import questionary
async def ask_prompts(prompt_json: dict) -> dict:
    """
    Docstring for ask_prompts
    
    :param prompt_json: Description
    :type prompt_json: dict
    """
    return {}